#!/bin/bash

# Set default umask
umask 022

DISTRO=${DISTRO:-$SUITE}
DISTRO=${DISTRO:?Must define distro}
build_config="${PWD}/${DISTRO}-build.sh"

# Use trusty for builds now
export USE_TRUSTY=${BUILD_WITH_TRUSTY:-0}

# Read in the common functions
my_dir="$( cd "$( dirname "$0" )" && pwd )"
base_dir=$(dirname ${my_dir})
source "${base_dir}/functions/locker"
source "${base_dir}/functions/common"
source "${base_dir}/functions/retry"
source "${my_dir}/build_lib.sh"
select_build_config

# Only block for serial if serial is unknown
[ -z "${SERIAL}" ] && {
    # Get the serial number
    retry 3 10 \
        "${base_dir}/get_serial.sh" \
             "${DISTRO}" "${WORKSPACE}" "${BUILD_ID}" "${BTYPE}" 0 0 1 1 ||
             fail "Failed to get serial for this build"

    # Get the serial number
    read SERIAL < serial.txt
    [ -z ${SERIAL} ] && echo "NO SERIAL" && exit 10
}

# Create the configurations
cmd=("${base_dir}/builder_config.sh"
     --distro "${DISTRO}"
     --build-type "${BTYPE}"
     --arch "${ARCH_TYPE}"
     --template ${base_dir}/templates/img-build.tmpl
     --serial "${SERIAL}"
     --out "${build_config}"
)

# Allow building from proposed
[ "${PROPOSED_BUILD:-0}" -eq 1 ] && cmd+=("--proposed")
[ "${USE_BUILDDS:-0}" -eq 1 ] && cmd+=("--bzr-automated-builds lp:~ubuntu-on-ec2/vmbuilder/automated-ec2-builds-buildd")
[ -n "${BZR_AUTOMATED_EC2}" ] && cmd+=("--bzr-automated-builds ${BZR_AUTOMATED_EC2}")
[ -n "${BZR_PUBSCRIPTS}" ] && cmd+=("--bzr-pubscripts ${BZR_PUBSCRIPTS}")
[ -n "${BZR_LIVEBUILD}" ] && cmd+=("--bzr-livebuild ${BZR_LIVEBUILD}")
[ -n "${BZR_VMBUILDER}" ] && cmd+=("--bzr-vmbuilder ${BZR_VMBUILDER}")

# Do the build
${cmd[@]} || fail "Failed to configure instance configuration"
unset cmd

# Exit after configuring for arm if so configured
if [[ "${ARCH_TYPE}" =~ (arm|aarch64|arm64) ]]; then
   echo "This is an ARM build. ARM rules will apply"
   [ "${BUILD_ARM}" -eq 0 ] && exit 0
fi

# Launch the builder
# Retry building the image twice, waiting five
# minutes. This should buffer most failures caused
# by bad mirrors.
export MAX_CYCLES=2160
retry 2 300 \
    "${base_dir}/launch_kvm.sh" \
         --id "${BUILD_ID}" \
         --user-data "${build_config}" \
         --cloud-config "${base_dir}/config/${cloud_init_cfg}" \
         --img-url "${BUILDER_CLOUD_IMAGE}" \
         --raw-disk "${WORKSPACE}/${DISTRO}.raw" \
         --raw-size 20 ||
         fail "KVM instance failed"

tar -xvvf "${WORKSPACE}/${DISTRO}.raw" ||
    fail "Result tar failed to unpack"

rm "${WORKSPACE}/${DISTRO}.raw" ||
    fail "Failed to remove unnecessary file"

# Put the bits in place
"${base_dir}/copy_to_final.sh" \
    "${DISTRO}" \
    "${WORKSPACE}" \
    "${SERIAL}" \
    "${BTYPE}" \
    "${TEST_BUILD}" \
    "${SANDBOX_BUILD}" \
    "${PROPOSED_BUILD}" ||
    fail "Failed to place final files to destination"

# Copy the build properties into the workspace. This is set by get_serial.sh
[ "${BUILD_PROPERTIES}" != "${WORKSPACE}/build_properties" ] &&
    cp ${BUILD_PROPERTIES} ${WORKSPACE}/build_properties

echo "ARCH=${ARCH_TYPE}" >> build_properties
